########################################################################
# File:    sessions_controller.rb                                      #
#          Hipposoft 2008                                              #
#                                                                      #
# Purpose: Manage OpenID logins. Originally created from examples in   #
#          the open_id_authentication plugin.                          #
#                                                                      #
# History: 06-Jan-2008 (ADH): Created.                                 #
########################################################################

class SessionsController < ApplicationController

  skip_before_filter( :appctrl_confirm_user,     :only => [ :new, :create           ] )
  skip_before_filter( :appctrl_ensure_user_name, :only => [ :new, :create, :destroy ] )

  # Called when the sign in form is submitted or when the OpenID plug-
  # in has completed a sign-in attempt, successfully or otherwise.

  def create
    identity_url = params[ :openid_url ]

    # Identity URL is *not* nil, but *is* empty? Form was submitted
    # with an empty string. URL is *not* nil and is *not* empty? Form
    # was submitted with a URL; call authentication routine. URL *is*
    # nil? We're being called from the Open ID plug-in with the result
    # of a sign-in attempt. Again, call the authentication routine.

    if ( not identity_url.nil? and identity_url.empty? )
      failed_login( 'You must provide an ID.')
    else
      identity_url = User.rationalise_id( identity_url )
      open_id_authentication()
    end

  rescue
    failed_login( 'ID appears to be malformed.' )
  end

  # Sign out - may be called by a normal user or if a user decides to
  # cancel during the sign-up process.

  def destroy
    user   = @current_user
    normal = ( @current_user and ( not @current_user.name.nil? ) and ( not @current_user.name.empty? ) )
    reset_session()

    if ( normal )
      flash[ :notice ] = 'You have signed out.'
    else
      user.destroy() if ( user )
      flash[ :error ] = 'Sign in process aborted.'
    end

    redirect_to( home_path() )
  end

protected

  def open_id_authentication()

    authenticate_with_open_id do | result, identity_url |
      identity_url = User.rationalise_id( identity_url )

      if result.successful?

        # The OpenID sign in went OK. If we can find an active user
        # with that ID, sign in is complete. If there's an inactive
        # user, complain. Otherwise, create a new user account - if
        # this is the first user, any OpenID will do; else it must
        # be in the permitted list.

        if @current_user = User.find_active_by_identity_url( identity_url )
          successful_login()

        else
          if ( User.find_inactive_by_identity_url( identity_url) )
            failed_login( "The account for OpenID '#{ identity_url }' has been deactivated. Please contact your system administrator for assistance." )

          else
            # Handle very first login auto-creation of the admin account

            if ( User.count == 0 )

              # Do this here because redirecting to the User controller
              # would require an exposed URL that could be used to try
              # and create users without OpenID authentication.

              user_type = 'Admin'

              @current_user = User.new
              @current_user.assign_defaults( nil, identity_url, user_type )
              @current_user.save!

              new_login()

            else

              # The identity URL does not match any existing user and the
              # administrator account already exists.

              failed_login( "Sorry, OpenID '#{ identity_url }' is not permitted to use this service. Please contact your system administrator for assistance.")

            end
          end
        end

      else
        # The OpenID login attempt failed.
        failed_login( result.message )

      end
    end
  end

private

  def successful_login
    flash[ :notice ] = 'Signed in successfully.'
    session[ :user_id ] = @current_user.id
    redirect_to( home_path() )
  end
  
  def new_login
    flash[ :notice ] = 'Signed in successfully. Please now fill in your timesheet system details.'
    session[ :user_id ] = @current_user.id
    redirect_to( edit_user_path( @current_user ) )
  end

  def failed_login( message )
    flash[ :error ] = message
    redirect_to( signin_path() )
  end
end
