########################################################################
# File:    reports_helper.rb                                           #
#          Hipposoft 2008                                              #
#                                                                      #
# Purpose: Support functions for views related to report generation.   #
#          See controllers/reports_controller.rb for more.             #
#                                                                      #
# History: 18-Feb-2008 (ADH): Created.                                 #
########################################################################

module ReportsHelper

  include TrackRecordReport

  # Helper class for ReportYear and reporthelp_week_selection.

  class ReportWeek
    def initialize( year, week )
      @id    = "#{ year }_#{ week }"
      @start = "#{ week } (starts Mon #{ Timesheet.date_for( year, week, TimesheetRow::FIRST_DAY ) })"
      @end   = "#{ week } (ends Sun #{ Timesheet.date_for( year, week, TimesheetRow::LAST_DAY ) })"
    end

    attr_reader( :start, :end, :id )
  end

  # Helper class for ReportYear and reporthelp_month_selection.

  class ReportMonth
    def initialize( year, month )
      @id    = "#{ year }_#{ month }"
      @start = "#{ month } (start of #{ Date::MONTHNAMES[ month ] } #{ year })"
      @end   = "#{ month } (end of #{ Date::MONTHNAMES[ month ] } #{ year })"
    end

    attr_reader( :start, :end, :id )
  end

  # Helper class for reporthelp_week_selection and reporthelp_month_selection.

  class ReportYear
    def initialize( year )
      @title  = year.to_s
      @weeks  = []
      @months = []

      # "limit" is set if the object is being built for the current year.
      # Weeks and months are checked and any that lie in the future are
      # not included.

      today = Date.current
      limit = ( year == today.year )
      range = ( 1..( Timesheet.get_last_week_number( year ) ) )

      range.to_a.reverse.each do | week |
        next if ( limit and Timesheet.date_for( year, week, TimesheetRow::FIRST_DAY, true ) > today )
        @weeks.push( ReportWeek.new( year, week ) )
      end

      ( 1..12 ).to_a.reverse.each do | month |
        next if ( limit and month > today.month )
        @months.push( ReportMonth.new( year, month ) )
      end
    end

    attr_reader( :title, :weeks, :months )
  end

  # Use the Calendar Date Select plug-in to generate a selector for
  # the start time of a report.

  def reporthelp_start_time
    return calendar_date_select(
      :report,
      :range_start,
      :embedded   => false,
      :year_range => Timesheet.time_range()
    )
  end

  # Use the Calendar Date Select plug-in to generate a selector for
  # the end time of a report.

  def reporthelp_end_time
    return calendar_date_select(
      :report,
      :range_end,
      :embedded   => false,
      :year_range => Timesheet.time_range()
    )
  end

  # Return HTML suitable for inclusion in a form which provides
  # a pull-down menu of years for the full permitted time range
  # subdivided into months. Pass the current item to select/nil
  # for none and 'true' for a start time menu, else an end time
  # menu.

  def reporthelp_month_selection( to_select, is_start )
    method = is_start ? :start : :end
    years  = []

    range = ( Timesheet.time_range().first() )..( Date.current.year )
    range.to_a.reverse.each do | year |
      years.push( ReportYear.new( year ) )
    end

    data = "<select id=\"report_range_month_#{method}\" name=\"report[range_month_#{method}]\">"
    data << '<option value="">-</option>'
    data << option_groups_from_collection_for_select(
      years,   # Use years for groups
      :months, # A years's "months" method returns its month list
      :title,  # Use the year 'title' for the group labels
      :id,     # A month's "id" method returns the value for an option tag
      method,  # A month's "start" or "end" method is used for the option contents
      to_select
    )
    return data << '</select>'
  end

  # Return HTML suitable for inclusion in a form which provides
  # a pull-down menu of years for the full permitted time range
  # subdivided into weeks. Pass 'true' for a start time menu,
  # else an end time menu.

  def reporthelp_week_selection( to_select, is_start )
    method = is_start ? :start : :end
    years  = []

    range = ( Timesheet.time_range().first() )..( Date.current.year )
    range.to_a.reverse.each do | year |
      years.push( ReportYear.new( year ) )
    end

    data = "<select id=\"report_range_week_#{method}\" name=\"report[range_week_#{method}]\">"
    data << '<option value="">-</option>'
    data << option_groups_from_collection_for_select(
      years,  # Use years for groups
      :weeks, # A years's "weeks" method returns its week list
      :title, # Use the year 'title' for the group labels
      :id,    # A week's "id" method returns the value for an option tag
      method, # A week's "start" or "end" method is used for the option contents
      to_select
    )
    return data << '</select>'
  end

  # Return HTML suitable for inclusion in the form passed in the
  # first parameter (i.e. the 'f' in "form for ... do |f|" ), which
  # provides a selection list allowing the user to choose a report
  # frequency (weekly, monthly etc.).

  def reporthelp_frequency_selection( form )
    collection = []

    Report.labels.each_index do | index |
      collection.push( [ Report.labels[ index ], index ] )
    end

    form.select( :frequency, collection )
  end

  # Return HTML suitable for inclusion in the form passed in the
  # first parameter (i.e. the 'f' in "form for ... do |f|" ), based
  # on the user array given in the second parameter, which provides:
  #
  # * A <select> tag with options listing all users in the array
  #   in the order in which they are stored in that array.
  #
  # * An empty string if the input users array is itself empty.

  def reporthelp_user_selection( form, users )
    if ( users.empty? )
      return ''
    else
      return apphelp_collection_select(
        form,
        'user_ids',
        users,
        :id,
        :name
      )
    end
  end

  # Pass a hash. The ":item" entry is read. If a User, a string of
  # "by <name>" is returned. Otherwise, a string of "on '<title>'" is
  # returned. Useful for indicating hours are done by a user, or were
  # worked on some task.

  def reporthelp_work_breakdown_item_name( item )
    item = item[ :item ]

    if ( item.class == User )
      return "by #{ h( item.name ) }"
    else
      return "on '#{ h( item.augmented_title ) }'"
    end
  end

  # Return HTML suitable for an 'hours' field which gives a total amount,
  # then a committed and not-committed amount wrapped in SPANs to give the
  # correct committed/uncommitted text styles. If the overall total time
  # is zero, "&nbsp;" is returned instead. Pass any object subclassing
  # ReportElementaryCalculator and, optionally, 'true' to show "0.0" rather
  # than "&nbsp;" in the zero hours total case.
  #
  # Instance variables @include_totals, @include_committed and
  # @include_not_committed are consulted in passing. If 'true' or 'nil',
  # the value is included. If explicitly 'false', the value is excluded.

  def reporthelp_hours( calculator, show_zero = false )

    if ( calculator.has_hours? )
      output  = ''
      output << apphelp_terse_hours( calculator.total ) if ( @include_totals != false )
      output << ' (' if ( @include_totals != false and ( @include_committed != false or @include_not_committed != false ) )

      if ( @include_committed != false )
        output << '<span class="timesheet_committed">'
        output << apphelp_terse_hours( calculator.committed )
        output << '</span>'
      end

      output << '/' if ( @include_committed != false and @include_not_committed != false )

      if ( @include_not_committed != false )
        output << '<span class="timesheet_not_committed">'
        output << apphelp_terse_hours( calculator.not_committed )
        output << '</span>'
      end

      output << ')' if ( @include_totals != false and ( @include_committed != false or @include_not_committed != false ) )

      return output
    else
      return ( show_zero ? '0' : '&nbsp;' )
    end
  end

  # Report a terse number of hours with an 'overrun' span if negative or a
  # 'no_overrun' span if >= 0 in two sections separated by a "/" - the idea
  # is to pass actual and potential remaining hours for a task in here.

  def reporthelp_decorated_hours( actual, potential )
    class_name  = actual    < 0 ? 'overrun' : 'no_overrun'
    output      = "<strong><span class=\"#{ class_name }\">#{ apphelp_terse_hours( actual ) }</span> / "
    class_name  = potential < 0 ? 'overrun' : 'no_overrun'
    output     << "<span class=\"#{ class_name }\">#{ apphelp_terse_hours( potential ) }</span></strong>"

    return output
  end

  # For a report 'show' view, generate a series of hidden fields that carry all
  # information about a report so that a 'new' view, or another report creation
  # operation, can progress with the same parameters as the current item. Pass
  # the wrapping form object reference ("f" in "form_for :report... do | f |").

  def reporthelp_hidden_fields( form )
    output  = form.hidden_field( :range_start       )
    output << form.hidden_field( :range_end         )
    output << form.hidden_field( :range_week_start  )
    output << form.hidden_field( :range_week_end    )
    output << form.hidden_field( :range_month_start )
    output << form.hidden_field( :range_month_end   )
    output << form.hidden_field( :frequency         )

    @report.users.each_index do | index |
      user = @report.users[ index ]
      output << hidden_field_tag( "report[user_ids][#{ index }]", user.id.to_s )
    end

    @report.tasks.each_index do | index |
      task = @report.tasks[ index ]
      output << hidden_field_tag( "report[task_ids][#{ index }]", task.id.to_s )
    end

    return output
  end
end
