/************************************************************************
 * File:    dynamic_task_list.js                                        *
 *          Hipposoft 2008                                              *
 *                                                                      *
 * Purpose: Replace a single static selection list of task names with a *
 *          series of dynamic selection lists breaking down selection   *
 *          by customer, project and task.                              *
 *                                                                      *
 *          Steps to use are:                                           *
 *                                                                      *
 *          1) Create a normal static task selection list, making sure  *
 *             that the list of tasks used is placed into @task_array.  *
 *             Optionally, include the list of initially selected tasks *
 *             in @task_selec.                                          *
 *                                                                      *
 *          2) Render partial shared/structured_task_list into a JS     *
 *             SCRIPT container. The partial outputs JS code.           *
 *                                                                      *
 *          3) Call replaceWithDynamicSelectors (see below).            *
 *                                                                      *
 * History: 06-Jun-2008 (ADH): Created.                                 *
 ************************************************************************/

function replaceWithDynamicSelectors( staticId, staticName )
{
    new DynamicTaskList( staticId, staticName );
}

/****************************************************************************\
 * LIST CONSTRUCTION
\****************************************************************************/

/* Replace a single SELECT list for task selection with a series of
 * dynamic lists, broken down into customers, projects and tasks. If
 * the selection in the customer list changes, the project list is
 * updated with all projects related to the selected customer present
 * and selected. In turn, changes to project selections are echoed in
 * a task selector.
 *
 * Pass the ID and name of the select list to replace. The final task
 * selector generated by the function will share the same ID and name.
 * You must render partial "shared/structured_taskInfo" to generate
 * the JS representation of the task list before calling.
 */

function DynamicTaskList( staticId, staticName )
{
    /* References to the three dynamically created SELECT lists */

    this.customerList = null;
    this.projectList  = null;
    this.taskList     = null;

    /* Task data related to each project added into the project list */

    this.projectIndexData = null;

    /* Find the selection list and remove it */

    var replaceMe = document.getElementById( staticId );
    var parent    = replaceMe.parentNode;

    parent.removeChild( replaceMe );

    try
    {
        /* Create a SELECT element */

        this.customerList          = document.createElement( 'select' );
        this.customerList.id       = 'dynamic_customer_list_' + staticId;
        this.customerList.multiple = true;

        parent.appendChild( this.customerList );
        addSeparator( parent );

        /* Create an alphabetically sorted list of items */

        var items = [];
        var itemsLength;

        for ( var customerId in taskInfo )
        {
            var customerTitle = taskInfo[ customerId ].title
            items.push( new ItemInfo( customerTitle, customerId ) );
        }

        items.sort( ItemInfo.sort );
        itemsLength = items.length;

        /* Add the OPTION elements */

        for ( var i = 0; i < itemsLength; i ++ )
        {
            var option = document.createElement( 'option' );

            option.value    = items[ i ].id;
            option.selected = customerIsSelected( items[ i ].id );
            option.appendChild( document.createTextNode( items[ i ].title ) );

            this.customerList.appendChild( option );
        }

        /* Set up the event listener */

        this.customerList.addEventListener( 'change', this, false );

        /* Create empty selection lists for the project and task, then call
         * helper functions for each to set them up in their default "empty"
         * state.
         */

        this.projectList          = document.createElement( 'select' );
        this.projectList.id       = 'dynamic_project_list_' + staticId;
        this.projectList.multiple = true;

        this.projectList.addEventListener( 'change', this, false );
        this.clearProjects();

        parent.appendChild( this.projectList );
        addSeparator( parent );

        this.taskList          = document.createElement( 'select' );
        this.taskList.id       = staticId;
        this.taskList.name     = staticName;
        this.taskList.multiple = true;

        this.clearTasks();

        parent.appendChild( this.taskList );

        /* In case any items were selected in the customer list, call the
         * on-change code now.
         */

        this.customersChanged( true );
    }
    catch( e )
    {
        /* If anything goes wrong, assume the browser doesn't understand some
         * of the code above and restore the original document instead.
         */

        removeChildren( parent );
        parent.appendChild( replaceMe );
    }
}

/****************************************************************************\
 * EVENT LISTENERS
\****************************************************************************/

/* Central event listener - call out to dedicated handler methods */

DynamicTaskList.prototype.handleEvent = function( event )
{
    if ( event.currentTarget == this.customerList ) this.customersChanged ( event );
    else                                            this.projectsChanged  ( event );
}

/* Handle selection changes in the list of customers. If passed 'true'
 * rather than an event payload, looks at the taskSel list to see which
 * of the projects in the project list generated based on the selected
 * customers should in turn be selected. Otherwise, everything is
 * selected.
 */

DynamicTaskList.prototype.customersChanged = function( eventOrBoolean )
{
    /* Read the current selection in order of added OPTION elements */

    var length      = this.customerList.options.length;
    var customerIds = [];

    for ( var i = 0; i < length; i ++ )
    {
        var option = this.customerList.options[ i ];
        if ( option.selected ) customerIds.push( option.value );
    }

    /* Empty the project list or populate it? */

    this.projectIndexData = [];

    if ( customerIds.length == 0 )
    {
        this.clearProjects();
        this.clearTasks();
    }
    else
    {
        removeChildren( this.projectList );
        this.projectList.disabled = false;

        length = customerIds.length;

        for ( var i = 0; i < length; i ++ )
        {
            var customerId    = customerIds[ i ];
            var customerTitle = taskInfo[ customerId ].title;
            var projects      = taskInfo[ customerId ].projects;

            /* Create an alphabetically sorted list of projects for each
             * of the customers.
             */

            var items = [];
            var itemsLength;

            for ( var projectId in projects )
            {
                var projectTitle = projects[ projectId ].title
                items.push( new ItemInfo( projectTitle, projectId ) );
            }

            items.sort( ItemInfo.sort );
            itemsLength = items.length;

            /* Add a disabled OPTION showing the customer name */

            appendOption( this.projectList, "Customer '" + customerTitle + "':" );
            this.projectIndexData.push( { customerId: -1, projectId: -1 } );

            /* Now add an OPTION element for each project */

            for ( var j = 0; j < itemsLength; j ++ )
            {
                var projectId   = items[ j ].id;
                var option      = document.createElement( 'option' );

                option.value    = projectId;
                option.selected = eventOrBoolean == true
                                ? projectIsSelected( customerId, projectId )
                                : true;

                option.appendChild( document.createTextNode( items[ j ].title ) );

                this.projectList.appendChild( option );
                this.projectIndexData.push( { customerId: customerId, projectId: projectId } );
            }
        }

        this.projectsChanged( eventOrBoolean );
    }

    this.resizeLists();
}

/* Handle selection changes in the list of projects; as with
 * customersChanged, pass 'true' if wishing to select items based on the
 * selected tasks in 'taskSel'.
 */

DynamicTaskList.prototype.projectsChanged = function( eventOrBoolean )
{
    var length            = this.projectIndexData.length;
    var selectedIndexData = [];

    for ( var i = 0; i < length; i ++ )
    {
        if ( this.projectList.options[ i ].selected ) selectedIndexData.push( this.projectIndexData[ i ] );
    }

    /* Empty the task list or populate it? */

    length = selectedIndexData.length;

    if ( length == 0 )
    {
        this.clearTasks();
    }
    else
    {
        removeChildren( this.taskList );
        this.taskList.disabled = false;

        for ( var i = 0; i < length; i ++ )
        {
            var indexData     = selectedIndexData[ i ];
            var customer      = taskInfo[ indexData.customerId ];
            var customerTitle = customer.title;
            var project       = customer.projects[ indexData.projectId ];
            var projectTitle  = project.title;
            var tasks         = project.tasks;
            var items         = []
            var itemsLength;

            /* Sort the task list by task code within each project */

            for ( var taskId in tasks )
            {
                var taskTitle = tasks[ taskId ].title
                var taskCode  = tasks[ taskId ].code

                items.push( new ItemInfo( taskTitle, taskId, taskCode ) );
            }

            items.sort( ItemInfo.sortByCode );
            itemsLength = items.length;

            /* Add a disabled OPTION showing the project name */

            appendOption( this.taskList, "Customer '" + customerTitle + "' - " + projectTitle + ":" );

            /* Now add an OPTION element for each task */

            for ( var j = 0; j < itemsLength; j ++ )
            {
                var taskId      = items[ j ].id;
                var taskTitle   = items[ j ].title;
                var option      = document.createElement( 'option' );

                option.value    = taskId;
                option.selected = eventOrBoolean == true
                                ? taskIsSelected( indexData.customerId, indexData.projectId, taskId )
                                : true;

                option.appendChild( document.createTextNode( taskTitle ) );
                this.taskList.appendChild( option );
            }
        }
    }

    this.resizeLists();
}

/****************************************************************************\
 * LIST MANAGEMENT
\****************************************************************************/

/* Helper function which clears out the project selection list */

DynamicTaskList.prototype.clearProjects = function()
{
    this.projectList.disabled = true;
    removeChildren( this.projectList );
    appendOption( this.projectList, 'Select a customer' );
}

/* Helper function which clears out the task selection list */

DynamicTaskList.prototype.clearTasks = function()
{
    this.taskList.disabled = true;
    removeChildren( this.taskList );
    appendOption( this.taskList, 'Select a project' );
}

/* Set the size of all SELECT lists to the maximum of the recommended
 * value for each. Individual list heights should not differ because
 * browsers are a pain and seem to steadfastly ignore all requests to
 * align the lists to their top edges, instead anchoring them at the
 * base, which looks terrible and makes item selection difficult since
 * (with mouse navigation) a list you just clicked in may move.
 */

DynamicTaskList.prototype.resizeLists = function()
{
    this.customerList.size = this.projectList.size = this.taskList.size = Math.max
    (
        sizeList( this.customerList ),
        sizeList( this.projectList  ),
        sizeList( this.taskList     )
    );

    /* Return a suggested value for the 'size' property of a given SELECT
     * list based on the number of OPTION elements within it.
     */

    function sizeList( list )
    {
        var items = list.options.length;

        if ( items < 3 ) items = 3;
        else if ( items > 30 ) items = 30;

        return items;
    }
}

/****************************************************************************\
 * DOCUMENT MANIPULATION
\****************************************************************************/

/* Helper function which adds a separator element to go between selection
 * lists to the given parent.
 */

function addSeparator( parent )
{
    parent.appendChild( document.createTextNode( ' ' ) );
}

/* Helper function which removes all children of the given parent */

function removeChildren( parent )
{
    while ( parent.firstChild ) parent.removeChild( parent.firstChild );
}

/* Helper function which creates a disabled OPTION tag, appending it to
 * the given parent, containing the given text.
 */

function appendOption( parent, text )
{
    var option      = document.createElement( 'option' );
    option.value    = '';
    option.disabled = true;
    option.appendChild( document.createTextNode( text ) );
    parent.appendChild( option );
}

/****************************************************************************\
 * SEARCH FUNCTIONS
\****************************************************************************/

/* Finder functions - search a given list ("this.taskList" or, more commonly,
 * "taskSel"), for a given ID. Followed by functions which find a
 * specific customer, project given a customer ID, or task given a
 * custmoer and project ID.
 */

function isSelected( list, findId )
{
    for( var id in list )
    {
        if ( id == findId ) return true;
    }

    return false;
}

/* See comments above */

function customerIsSelected( findCustomerId )
{
    return isSelected( taskSel, findCustomerId );
}

/* See comments above */

function projectIsSelected( customerId, findProjectId )
{
    return isSelected( taskSel[ customerId ].projects, findProjectId );
}

/* See comments above */

function taskIsSelected( customerId, projectId, findTaskId )
{
    return isSelected( taskSel[ customerId ].projects[ projectId ].tasks, findTaskId );
}

/****************************************************************************\
 * INFORMATION ENCAPSULATION
\****************************************************************************/

/* Small object which stores title, ID and optional code for an item */

ItemInfo = function( title, id, code )
{
    this.title = title;
    this.id    = id;
    this.code  = code;
}

/* Class function which can be passed to Array.sort() - sorts by title */

ItemInfo.sort = function( a, b )
{
    if      ( a.title < b.title ) return -1;
    else if ( a.title > b.title ) return 1;
    else return 0;
}

/* Class function which can be passed to Array.sort() - sorts by code */

ItemInfo.sortByCode = function( a, b )
{
    if      ( a.code < b.code ) return -1;
    else if ( a.code > b.code ) return 1;
    else return 0;
}
